/*
   Copyright(C) 1998
   All Rights Reserved. Salih Yurttas, Computer Science, TAMU.

   Permission to use, copy, modify, and distribute this
   software and its documentation for EDUCATIONAL purposes
   and without fee is hereby granted provided that this
   copyright notice appears in all copies.

   date   : January 1, 1998.
   author : Salih Yurttas.

   Matrix.cpp
*/


#include <iostream>

using namespace std;

#include "Matrix.h"

Matrix::
Matrix(const int r, 
       const int c) : r_size(r),
                      c_size(c),
                      m(new int*[r_size]) {
  for(int i=0; i<r_size; ++i) {
    m[i]= new int[c_size];
    for(int j=0; j<c_size; ++j)
      m[i][j] = 0;
  }
}

Matrix::
Matrix(const Matrix& m1) : r_size(m1.r_size),
                           c_size(m1.c_size),
                           m(new int*[r_size]) {
  for(int i=0; i<r_size; ++i) {
    m[i]= new int[c_size];
    for(int j=0; j<c_size; ++j)
      m[i][j] = m1.m[i][j];
  }
}

Matrix::
~Matrix() {
  for(int i=0; i<r_size; ++i) {
    delete [] m[i];
    m[i] = NULL;
  }
  delete [] m;
  m = NULL;
  r_size = 0;
  c_size = 0;
}

Matrix&
Matrix::
operator=(const Matrix& m1) {
  if(this==&m1) return *this;

  for(int i=0; i<r_size; ++i)
    delete [] m[i];
  delete [] m;

  r_size = m1.r_size;
  c_size = m1.c_size;

  m = new int*[r_size];

  for(int i=0; i<r_size; ++i) {
    m[i]= new int[c_size];
    for(int j=0; j<c_size; ++j)
      m[i][j] = m1.m[i][j];
  }

  return *this;
}

Matrix&
Matrix::
operator+=(const Matrix& m1) {
  for(int i=0; i<r_size; ++i)
    for(int j=0; j<c_size; ++j)
      m[i][j] += m1.m[i][j];

  return *this;
}

Matrix&
Matrix::
operator-=(const Matrix& m1) {
  for(int i=0; i<r_size; ++i)
    for(int j=0; j<c_size; ++j)
      m[i][j] -= m1.m[i][j];

  return *this;
}

Matrix&
Matrix::
operator*=(const Matrix& m1) {
  for(int i=0; i<r_size; ++i)
    for(int j=0; j<m1.c_size; ++j) {
      m[i][j] = 0;
      for(int k=0; k<c_size; k++)
        m[i][j] += m[i][k] * m1.m[k][j];
    }

  return *this;
}

Matrix
operator+(const Matrix& m1,
          const Matrix& m2) {
  Matrix t(m1.r_size,
           m1.c_size);

  for(int i=0; i<m1.r_size; ++i)
    for(int j=0; j<m1.c_size; ++j)
      t.m[i][j] = m1.m[i][j] + m2.m[i][j];

  return t;
}

Matrix
operator-(const Matrix& m1,
          const Matrix& m2) {
  Matrix t(m1.r_size,
           m1.c_size);

  for(int i=0; i<m1.r_size; ++i)
    for(int j=0; j<m1.c_size; ++j)
      t.m[i][j] = m1.m[i][j] - m2.m[i][j];

  return t;
}

Matrix
operator*(const Matrix& m1,
          const Matrix& m2) {
  Matrix t(m1.r_size,
           m1.c_size);

  for(int i=0; i<m1.r_size; ++i)
    for(int j=0; j<m2.c_size; ++j) {
      t.m[i][j] = 0;
      for(int k=0; k<m1.c_size; k++)
        t.m[i][j] += m1.m[i][k] * m2.m[k][j];
    }

  return t;
}

Matrix
operator*(const Matrix& m1,
          const int s) {
  Matrix t(m1.r_size,
           m1.c_size);

  for(int i=0; i<m1.r_size; ++i)
    for(int j=0; j<m1.c_size; ++j)
      t.m[i][j] = m1.m[i][j] * s;

  return t;
}

Matrix
operator*(const int s,
          const Matrix& m1) {
  Matrix t(m1.r_size,
           m1.c_size);

  for(int i=0; i<m1.r_size; ++i)
    for(int j=0; j<m1.c_size; ++j)
      t.m[i][j] = s * m1.m[i][j];

  return t;
}

int
Matrix::
retrieve_element(const int r,
                 const int c) const {
  return m[r][c];
}

void
Matrix::
assign_element(const int r,
               const int c,
               const int value) {
  m[r][c] = value;
}

void
Matrix::
negate() {
  for(int i=0; i<r_size; ++i)
    for(int j=0; j<c_size; ++j)
      m[i][j] *= -1;
}

void
Matrix::
put_matrix() const {
  for(int i=0; i<r_size; ++i) {
    for(int j=0; j<c_size; ++j)
      cout << m[i][j] << ' ';
    cout << endl;
  }
}
